Index: src/ia32/lithium-codegen-ia32.cc |
diff --git a/src/ia32/lithium-codegen-ia32.cc b/src/ia32/lithium-codegen-ia32.cc |
index ec87a9c72ed88b442efa294297e25d551426a076..fd0e270aeca8d3ff895918d02a41c3c09252f80e 100644 |
--- a/src/ia32/lithium-codegen-ia32.cc |
+++ b/src/ia32/lithium-codegen-ia32.cc |
@@ -2134,13 +2134,25 @@ void LCodeGen::DoBranch(LBranch* instr) { |
ASSERT(!info()->IsStub()); |
__ test(reg, Operand(reg)); |
EmitBranch(true_block, false_block, not_equal); |
+ } else if (type.IsJSArray()) { |
+ ASSERT(!info()->IsStub()); |
+ EmitBranch(true_block, false_block, no_condition); |
+ } else if (type.IsHeapNumber()) { |
+ ASSERT(!info()->IsStub()); |
+ CpuFeatureScope scope(masm(), SSE2); |
+ __ xorps(xmm0, xmm0); |
+ __ ucomisd(xmm0, FieldOperand(reg, HeapNumber::kValueOffset)); |
+ EmitBranch(true_block, false_block, not_equal); |
+ } else if (type.IsString()) { |
+ ASSERT(!info()->IsStub()); |
+ __ cmp(FieldOperand(reg, String::kLengthOffset), Immediate(0)); |
+ EmitBranch(true_block, false_block, not_equal); |
} else { |
Label* true_label = chunk_->GetAssemblyLabel(true_block); |
Label* false_label = chunk_->GetAssemblyLabel(false_block); |
ToBooleanStub::Types expected = instr->hydrogen()->expected_input_types(); |
- // Avoid deopts in the case where we've never executed this path before. |
- if (expected.IsEmpty()) expected = ToBooleanStub::all_types(); |
+ if (expected.IsEmpty()) expected = ToBooleanStub::Types::Generic(); |
if (expected.Contains(ToBooleanStub::UNDEFINED)) { |
// undefined -> false. |
@@ -2229,8 +2241,11 @@ void LCodeGen::DoBranch(LBranch* instr) { |
__ bind(¬_heap_number); |
} |
- // We've seen something for the first time -> deopt. |
- DeoptimizeIf(no_condition, instr->environment()); |
+ if (!expected.IsGeneric()) { |
+ // We've seen something for the first time -> deopt. |
+ // This can only happen if we are not generic already. |
+ DeoptimizeIf(no_condition, instr->environment()); |
+ } |
} |
} |
} |